@@ -98,6 +98,9 @@ class alti_data(htools.hydro_data) :
            
         '''
         
+        self.datatype=None
+        self.sat=[]
+        
         if time_range is not None :
             ls=np.array(glob.glob(file_pattern))
             if len(ls) == 0: raise Exception('File not found : {0}'.format(file_pattern))
@@ -131,7 +134,7 @@ class alti_data(htools.hydro_data) :
         set satellite name using (cf. notes on file names in `altimetry.data.alti_data.__init__`)
         '''
         if self.count == 0 : return
-        self.sat=[]
+        
         for enum in enumerate(self.filelist):
             
             if hasattr(self, 'id') : sat = self.id
@@ -220,8 +223,12 @@ class alti_data(htools.hydro_data) :
         
         #Gat sat name
         splitted=os.path.basename(filename).split(delim)
-        if (datatype == 'DT') | (datatype == 'NRT') : sat_name = splitted[2] if splitted[0] == 'nrt' else splitted[3]
-        if datatype == 'PISTACH' : sat_name = 'J2'
+        if len(splitted) > 3 :
+	    if (datatype == 'DT') | (datatype == 'NRT') : sat_name = splitted[2] if splitted[0] == 'nrt' else splitted[3]
+	    elif datatype == 'PISTACH' : sat_name = 'J2'
+	    else : sat_name = 'J2'
+	else :
+	    sat_name="N/A"
         
      
         #Get list of recorded parameters:
@@ -380,7 +387,12 @@ class alti_data(htools.hydro_data) :
         
         for i in ['DeltaT','NbPoints','Cycles','Tracks','DataIndexes'] : par_list.pop(par_list.index(i))
         
-        outStr={'_dimensions':dimStr,'lon':lon,'lat':lat,'date':date,'cycle':cycles,'track':tracks}
+        outStr={'_dimensions':dimStr,
+                'lon':lon,
+                'lat':lat,
+                'date':date,
+                'cycle':cycles,
+                'track':tracks}
         
         
         
@@ -498,7 +510,7 @@ class alti_data(htools.hydro_data) :
             exec(cmd)
             outStr.update(dumStr)
         
-        id={'data':np.repeat(sat_name,sz)}
+        id=np.repeat(sat_name,sz)
         
         
         outStr.update({'id':id})
@@ -554,7 +566,7 @@ class alti_data(htools.hydro_data) :
         nlon = len(lonout)
         
         #Get cycles
-        cycle = self.load_ncVar('cycle',**kwargs)
+        cycle = self.load_ncVar('cycle',time=ind,**kwargs)
         cycleout = cycle.pop('data')
         ncycles = len(cycleout)
         
@@ -579,7 +591,7 @@ class alti_data(htools.hydro_data) :
         
         cycle={'_dimensions':{'_ndims':1,'time':N},
                '_attributes':cycle['_attributes'],
-             'data':np.ma.masked_array(np.repeat(cycleout.data,nlon),mask=mask),
+             'data':np.ma.masked_array(np.repeat(cycleout.data,nlon).reshape((ncycles,nlon)).T.flatten(),mask=mask),
              'long_name':'cycle_number'}
         
         track={'_dimensions':{'_ndims':1,'time':N},
@@ -645,8 +657,9 @@ class alti_data(htools.hydro_data) :
         #Remove attributes already existing in data object
         for a in self.__dict__.keys():
             if outStr.has_key(a) and not a.startswith('_') :
-                if remove_existing : outStr.pop(a)
-                self.message(4, 'Attribute {0} already exists'.format(a))
+                if remove_existing :
+                    outStr.pop(a)
+                    self.message(2, 'Attribute {0} already exists - removing it (set remove_existing to False instead)'.format(a))
         
         return outStr
 
