@@ -11,15 +11,17 @@ import matplotlib.pyplot as plt
 from netCDF4 import Dataset as ncfile
 import copy
 
-from altimetry.tools.nctools import load_ncVar, load_ncVar_v2, nc as ncobj
+from altimetry.tools.nctools import load_ncVar, load_ncVar_v2, nc as ncobj, dimStr,attrStr
 try:
     import seawater.csiro as csw
 except ImportError:
-    warn("Module seawater doesn't exists. not loading it")
+    warn("[WARNING:%s] Module seawater doesn't exists. not loading it" % __name__)
     pass # module doesn't exist, deal with it.
 
 #import alti_tools as atools
-from scipy import interpolate
+try : from scipy import interpolate
+except ImportError :
+    warn("[WARNING:%s] module scipy not found" % __name__)
 
 from altimetry.tools import recale_limits, in_limits, cumulative_distance, calcul_distance, \
     where_list, \
@@ -94,11 +96,24 @@ class hydro_data(object):
         if isinstance(file_pattern, str) : ls=glob.glob(file_pattern)
         else :
             ls = file_pattern.tolist() if not isinstance(file_pattern,list) else file_pattern
-            file_pattern=file_pattern[0]
+            file_pattern=file_pattern[0] if len(file_pattern) > 0 else []
         
         if len(ls) == 0 :
-            self.Error('File pattern not matched : '+file_pattern)
-                
+            if isinstance(file_pattern, str) :
+                self.Error('File pattern not matched '+file_pattern)
+            if isinstance(file_pattern, list) :
+                self.warning(2, 'Empty file list - will return empty object')
+        
+        self._filename=None
+        '''
+        Name of the file currently used
+        '''
+        
+        self._ncfile=None
+        '''
+        Name of the file currently used
+        '''
+        
         self.filelist=[os.path.basename(j) for i,j in enumerate(ls)]
         '''
         list of files being loaded
@@ -112,12 +127,14 @@ class hydro_data(object):
         enum = list(enumerate(ls))
         enum = zip(*enum)
         
-        self.fid_list=np.array(enum[0])
+        self.fid_list=np.array(enum[0]) if len(enum) > 0 else np.array([])
         
-        self.dirname=os.path.dirname(file_pattern)
+        self.dirname=os.getcwd()
         '''
-        Directory name of the file pattern being globbed (:func:`glob.glob`)
+        Directory name of the file pattern being globbed (:func:`glob.glob`). Defaulted to current directory
         '''
+        if isinstance(file_pattern,str) : self.dirname=os.path.dirname(os.path.abspath(file_pattern))
+        elif len(file_pattern) > 0 : self.dirname=os.path.dirname(os.path.abspath(file_pattern[0]))  
         
         self.par_list=np.array([])
         '''
@@ -129,7 +146,7 @@ class hydro_data(object):
         array containing the dimensions of each parameter
         '''
         
-        self._dimensions={'_ndims':0}
+        self._dimensions=dimStr()
         '''
         dimensional strucutre
         '''
@@ -143,8 +160,6 @@ class hydro_data(object):
             filename = enum[1][i]
             self.message(1,"Loading "+os.path.basename(filename))
             
-            self.current_file=enum[0][i]
-            
             res=self.read(filename,output_is_dict=output_is_dict,**kwargs) #read() function is specific of each class
             self.update_dataset(res,flatten=flatten) #update class with loaded data
             
@@ -179,14 +194,18 @@ class hydro_data(object):
         isStructure = True if isinstance(dataStr[keys[0]],dict) else False
         
 #        datalen = [np.size(dataStr[key]) for key in keys]
-        datalen = [list(np.shape(dataStr[key]['data'])) for key in keys] if isStructure else [list(np.shape(dataStr[key])) for key in keys] #####!!!!!! WARNING!!!! NEW FEATURE TO TEST
+        datalen = [list(np.shape(dataStr[key]['data'])[::-1]) for key in keys] if isStructure else [list(np.shape(dataStr[key])[::-1]) for key in keys] #Shape is inverted wrt to order of dimensions to be consistent with check_variable
         if isStructure :
-            varDim = [list(dataStr[key]['_dimensions'])[1:][::-1] for key in keys]
+            varDim = [list(dataStr[key]['_dimensions'])[1:] for key in keys]
             ind = [where_list(vDim,dimensions[0]) for vDim in varDim] #Dimensions indices from actual variables' dimensions
+            
             #Check dimension lengths
-            dimOk = np.array([enum[1][0] == dimensions[1][ind[enum[0]][0]] for enum in enumerate(datalen)])
-            if (~dimOk).sum() > 0 : 
-                notOk = np.where(~dimOk)[0]
+#             dimOk = np.array([enum[1][0] == dimensions[1][ind[enum[0]][0]] for enum in enumerate(datalen)])
+            dimOk = [any([enum[1][ii] == dimensions[1][jj] for ii,jj in enumerate(ind[enum[0]])]) for enum in enumerate(datalen)]
+            
+
+            if any([not d for d in dimOk]) : 
+                notOk = np.where(~np.array(dimOk))[0]
                 self.Error('Problem with {0} variables : {1}'.format(len(notOk),','.join(np.array(dataStr.keys())[notOk])))
         else :
             ind = [where_list(dlen,dimensions[1]) for dlen in datalen] #Dimensions indices from variable length
@@ -206,6 +225,7 @@ class hydro_data(object):
         
         self.message(2, 'Updating object with '+str(['{0}({1}:{2})'.format(i[0],i[1],i[2]) for i in zip(*(keys,dimname,datalen))]))
         
+        #Update variables available in files
         for enum in enumerate(keys) :
             
             ind=enum[0]
@@ -214,14 +234,24 @@ class hydro_data(object):
             #Load variable
             ##############
 #            var=dataStr.get(key)
-            dum=dataStr.get(key).get('data') if isStructure else dataStr.get(key) 
+            dum=dataStr.get(key).pop('data') if isStructure else copy.deepcopy(dataStr.get(key)) 
             if flatten :
                 if isinstance(dum,dict) :dum['data']=dum['data'].flatten()
                 else : dum=dum.flatten()
             
-            dum={'data':dum}
-            if dataStr[key].has_key('_attributes'):
-                dum.update(dataStr[key]['_attributes'])
+            if not isStructure :
+                dum={'_dimensions':dum._dimensions if hasattr(dum,'_dimensions') else {},
+                      '_attributes':dum._attributes if hasattr(dum,'_attributes') else {},
+                      'data':dum}
+            else :
+                dumStr=dataStr.get(key)
+                dumStr.update({'data':dum})
+                dum=dumStr
+
+            dumDim=dimStr(dimname[ind],datalen[ind])
+
+#             if dataStr[key].has_key('_attributes'):
+#                 dum.update(dataStr[key]['_attributes'])
 #            if isinstance(dum,np.ma.masked_array) : 
             
 #            #Get associated dimensions
@@ -234,9 +264,16 @@ class hydro_data(object):
             #Initialize variable if required
 #            if toCreate :
 #            updateDim.append(self.create_Variable(key, dum, dimensions={dimname[ind]:datalen[ind]},toCreate=toCreate[ind],createDim=createDim[ind]))
-            updateDim.append(self.create_Variable(key, dum, dimensions=dict(zip(dimname[ind],datalen[ind])),toCreate=toCreate[ind],createDim=createDim[ind]))
+            updateDim.append(self.create_Variable(key, dum, dimensions=dumDim,toCreate=toCreate[ind],createDim=createDim[ind]))
 
         
+        #Extend missing variables
+#         missing__keys = list(set(self.par_list).difference(keys))
+#         for enum in enumerate(missing__keys) :
+#             ind=enum[0]
+#             key=enum[1]
+#             updateDim.append(self.create_Variable(key, np.ma.repeat(self.dist_to_coast_leuliette.fill_value), dimensions=dumDim,toCreate=False,createDim=False))
+        
         
         #Final sequence
         
@@ -332,13 +369,30 @@ class hydro_data(object):
         self.count = len(self.fileid)
         self.size = np.size([np.size(self.__dict__.get(par)) for par in self.par_list])
         
-        infos = zip(*(self.par_list,self.dim_list))
+        infos = zip(*(self.par_list.tolist(),self.dim_list.tolist()))
 #        curDim, nself = self.get_currentDim()
         
         for enum in enumerate(infos):
             varSize = np.size(self.__dict__.get(enum[1][0]))
             varShape = np.shape(self.__dict__.get(enum[1][0]))[::-1] #Data and Netcdf dimensions are inverted (not allways?)
-            dimSize = tuple([(self._dimensions).get(d) for d in enum[1][1]])
+            
+            dimSize = None
+            if hasattr(self.__dict__.get(enum[1][0]),'_dimensions') :
+                dumDim = self.__dict__.get(enum[1][0])._dimensions
+                dumDim.pop('_ndims')
+                dimSize = tuple(dumDim.values())
+            elif isinstance(self.__dict__.get(enum[1][0]),dict):
+                if self.__dict__.get(enum[1][0]).has_key('_dimensions'):
+                    dumDim = self.__dict__.get(enum[1][0])['_dimensions']
+                    dumDim.pop('_ndims')
+                    dimSize = tuple(dumDim.values())
+            
+            if dimSize is None :
+                if isinstance(self._dimensions,dimStr) :
+                    dimSize = (self._dimensions).get(enum[1][1])
+                else :
+                    dimSize = tuple([(self._dimensions).get([d]) for d in enum[1][1]])
+                
             masked = isinstance(self.__dict__.get(enum[1][0]), np.ma.masked_array)
             
             #Check mask consistency (mask length should be the same as data)
@@ -353,8 +407,14 @@ class hydro_data(object):
                 elif (sh < dimSize[n]):
                     self.message(3, 'Variable {0}(N={1}) being extended to match dimension {2}:{3}'.format(enum[1][0],varSize,enum[1][1],dimSize))  
     #                self.__dict__[enum[1][0]] = np.ma.concatenate((self.__dict__[enum[1][0]], np.ma.masked_array(np.repeat(np.nan,dimSize - varSize),mask=np.zeros(dimSize - varSize,dtype='bool'))))
+    
+                    #Save additonnal attributes
+                    attrStr={}
+                    for a in set(self.__dict__[enum[1][0]].__dict__.keys()).difference(np.ma.empty(0,dtype=bool).__dict__.keys()):
+                        attrStr.update({a:self.__dict__[enum[1][0]].__dict__[a]})
                     self.__dict__[enum[1][0]] = np.ma.masked_array( np.append(self.__dict__[enum[1][0]].data,np.repeat(self.__dict__[enum[1][0]].fill_value if hasattr(self.__dict__[enum[1][0]],'fill_value') else np.NaN,dimSize[n] - varSize)),
                                                                     mask=np.append(self.__dict__[enum[1][0]].mask,np.ones(dimSize[n] - varSize,dtype='bool')) )
+                    self.__dict__[enum[1][0]].__dict__.update(attrStr)
     
     def create_Dim(self, name,value):
         '''
@@ -380,6 +440,13 @@ class hydro_data(object):
         oldVal=self._dimensions[name]
         self._dimensions[name] += value
         self.message(2, 'Updating dimension {0} (from {1} to {2})'.format(name,oldVal,self._dimensions[name]))
+        
+        #Update dimensions within all variables
+        for p in self.par_list:
+            if self.__dict__[p].__dict__.has_key('_dimensions'):
+                for d in self.__dict__[p]._dimensions.keys():
+                    self.__dict__[p]._dimensions.update({d:self._dimensions[d]})
+        
     
     def update_fid_list(self,filename,N):
         '''
@@ -495,20 +562,21 @@ class hydro_data(object):
         #Cast variable into masked array first
         ######################################
         if (not isinstance(value['data'],np.ma.core.MaskedArray) if isStructure else not isinstance(value,np.ma.core.MaskedArray)) :
-            value = np.ma.masked_array(value['data'],mask=np.zeros(tuple(dimVal),dtype='bool')) if isStructure else np.ma.masked_array(value,mask=np.zeros(tuple(dimVal),dtype='bool'))
+            value['data'] = np.ma.masked_array(value['data'],mask=np.zeros(tuple(dimVal),dtype='bool')) if isStructure else np.ma.masked_array(value,mask=np.zeros(tuple(dimVal),dtype='bool'))
             self.message(4,'Casting variable to np.ma.MaskedArray')
         
         #Restructure dataset if structure
         if isStructure :
             dumvalue=value.pop('data')
-            for a in value.keys() :
-                dumvalue.__setattr__(a,value[a])
-            value=dumvalue
+            if value.has_key('_attributes'):
+                for a in value['_attributes'].keys():
+                    self.message(4, "copying attribute %s" % a)
+                    dumvalue.__setattr__(a,value['_attributes'][a])
+            value=copy.deepcopy(dumvalue)
         
         
         curDim, nself=self.get_currentDim()
         
-        
         curInd=np.array(where_list(dimName,curDim[0]))
         curDimVal=np.array(where_list(dimVal,curDim[1]))
         
@@ -533,9 +601,11 @@ class hydro_data(object):
         # 2: extend -> create a new variable using existing dimensions
         # 3: append exisiting variable with data
         # 4: impossible case ?
+        
+        #1) Create variable
         if createDim.any() & toCreate :           
             #Create Variable
-            self.message(4,'Create variable '+name)
+            self.message(4,'Create variable %s '+name)
 #            self.__setattr__(name,value)
 #            cmd='self.'+name+'=value'
 
@@ -547,10 +617,15 @@ class hydro_data(object):
         
             updateDim=False
         
+        #2) Extend
         elif (not createDim.any()) & toCreate :
             
             #extend variable
-            if extend : value = np.ma.masked_array(np.append(np.zeros(curDim[1][curInd]),value.data),mask=np.append(np.ones(curDim[1][curInd],dtype='bool'),value.mask))
+            if extend :
+                dumvalue = np.ma.masked_array(np.append(np.zeros(curDim[1][curInd]),value.data),mask=np.append(np.ones(curDim[1][curInd],dtype='bool'),value.mask))
+                for a in set(value.__dict__.keys()).difference(dumvalue.__dict__.keys()) :
+                    dumvalue.__setattr__(a,value.__dict__[a] if hasattr(value, a) else self.__getattribute__(name).__getattribute__(a))
+                value=copy.deepcopy(dumvalue)
             
             self.message(4,'Extend variable '+name)
 #            self.__setattr__(name,value)
@@ -566,27 +641,45 @@ class hydro_data(object):
             
             updateDim=True
         
+        #3) Append
         elif (not createDim.any()) & (not toCreate) :
             #append variable
             self.message(4,'Append data to variable '+name)
-            value = np.ma.masked_array(np.append(self.__getattribute__(name).data,value.data),mask=np.append(self.__getattribute__(name).mask,value.mask))
-#             cmd = 'self.'+name+'=np.ma.concatenate((self.'+name+',value),mask=np.append(self.dac))' #buggy command : concatenate do not always work..
-#            cmd = 'self.{0}=np.ma.masked_array(np.append(self.{0}.data,value.data),mask=np.append(self.{0}.mask,value.mask))'.format(name)
-        
-#            exec('res = [self.{0}.mask.size,value.mask.size]'.format(name))
-#                raise 'error!'
-        
+            
+            dumvalue = np.ma.masked_array(np.append(self.__getattribute__(name).data,value.data),mask=np.append(self.__getattribute__(name).mask,value.mask))
+            
+            #We gather a list of attributes :
+            # - already in data structure,
+            # - in current data file
+            # - and not in output structure
+            attributes=set(self.__getattribute__(name).__dict__.keys())
+            attributes=attributes.union(value.__dict__.keys())
+#             attributes=attributes.difference(self.__getattribute__(name).__dict__.keys())
+            attributes=attributes.difference(dumvalue.__dict__.keys())
+            
+            #Then :
+            # - we add attributes of current file not in data structure
+            # - we keep attributes of current data structure if they exist 
+            for a in attributes :
+                dumvalue.__setattr__(a,value.__dict__[a] if hasattr(value, a) else self.__getattribute__(name).__getattribute__(a))
+            value=copy.deepcopy(dumvalue)
+                    
             updateDim=True
         
         elif createDim.any() & (not toCreate) :
             #Impossible case ?
             self.Error('Impossible case : create dimensions and variable {0} already existing'.format(name))
         
+        
         #Append dimensions to variable
         if not dimensions.has_key('_ndims') :
-            dumDim={'_ndims':len(dimensions.keys())}
-            dumDim.update(dimensions)
+            dumDim=dimStr(dimensions)
             dimensions=dumDim.copy()
+        
+        #Update variable dimensions
+        if updateDim :
+            for k in dimensions.keys(): dimensions.update({k:self._dimensions[k]})
+        
         value.__setattr__('_dimensions',dimensions)
         
         try : self.__setattr__(name,value)
@@ -631,10 +724,12 @@ class hydro_data(object):
         returns the current dimensions of the object
         '''
         selfDim = self._dimensions.copy()
-        if selfDim.has_key('_ndims') : nself = selfDim.pop('_ndims')
-        else : 
-            self.warning(1, 'self._dimensions does not have the _ndims key')
-            nself = len(selfDim)
+        if not isinstance(selfDim,dimStr):
+            if selfDim.has_key('_ndims') : nself = selfDim.pop('_ndims')
+            else : 
+                self.warning(1, 'self._dimensions does not have the _ndims key')
+                nself = len(selfDim)
+        else : nself = selfDim['_ndims']
         curDim = [[key for key in selfDim.keys()],[selfDim[key] for key in selfDim.keys()]]
         return curDim, nself
     
@@ -685,13 +780,63 @@ class hydro_data(object):
         '''
         raise Exception(ErrorMsg)
     
-    def copy(self,deep=True):
+    def updated_copy(self,flag,deep=True):
+        '''
+        Returns a sliced (updated) copy of current data object
+        
+        :summary: This has the same effect as `obj.copy();obj.update(flag)` but is much less memory consumming.
+        
+        .. note:: TypeError could arise if some object attributes are setted outside the :func:`__init__` function (eg. for data objects derived from :class:`hydro_data`). If this is the case, initialise these attributes within their respective :func:`__init__`.
+        
+        '''
+        
+        
+        emptyObj=self.__class__([])
+        
+        if deep : func_copy = copy.deepcopy
+        else : func_copy = copy.copy
+        
+        #Get object attributes from self
+        for a in set(self.__dict__.keys()).intersection(emptyObj.__dict__.keys()):
+            self.message(4, 'updating attribute %s to returned object' % a)
+            emptyObj.__dict__[a] = func_copy(self.__dict__[a])
+        
+        #Get slices
+        for p in set(self.__dict__.keys()).difference(emptyObj.__dict__.keys()):
+            self.message(4, 'updating parameter %s to returned object' % p)
+            try :
+                #Save additionnal attributes
+                attrlist=list(set(dir(self.__dict__[p])).difference(dir(np.ma.array(0))))
+                attrvalues=[self.__dict__[p].__dict__[l] for l in attrlist]
+                attr=attrStr(attrlist,attrvalues)
+                emptyObj.__dict__[p] = func_copy(self.__dict__[p][flag])
+                for a in attr.keys(): setattr(emptyObj.__dict__[p], a, attr[a])
+            except TypeError:
+                self.warning(1,'Could not slice %s - check if this attribute is in self.__init__')
+            
+        #update dimensions
+        N=flag.sum()
+        emptyObj._dimensions['time']=N
+        emptyObj.count=N
+        
+#         self.message(4, "Checking variables consistency")
+#         emptyObj.check_variables()
+        
+        return emptyObj
+    
+    def copy(self,*args,**kwargs):
         '''
         Returns a copy of the current data object
         
-        :keyword deep: deep copies the object (object data will be copied as well).
+        :param flag: if an argument is provided, this returns an updated copy of current object (ie. equivalent to obj.copy();obj.update(flag)), optimising the memory (
+        :keyword True deep: deep copies the object (object data will be copied as well).
         '''
-        return copy.deepcopy(self) if deep else copy.copy(self)
+        deep=kwargs.get('deep',True)
+        
+        if len(args) > 0:
+            return self.updated_copy(*args)
+        else :
+            return copy.deepcopy(self) if deep else copy.copy(self)
     
     def slice(self,param,range,surf=False):
         '''
@@ -830,14 +975,19 @@ class hydro_data(object):
         avail_par = par_list.compress(per_valid > 0)
         avail_par_per = per_valid.compress(per_valid > 0)
         
-        return par_list, valid, per_valid, fname, trange, extent, N, avail_par, avail_par_per
+        par_stats = OrderedDict()
+        for p in avail_par :
+            var = self.__dict__[p].compress(flag).compressed()
+            par_stats.update({p:{'mean':np.mean(var),'min':np.min(var),'max':np.max(var),'ptp':np.ptp(var),'std':np.std(var),'median':np.median(var)}})
+        
+        return par_list, valid, per_valid, fname, trange, extent, N, avail_par, avail_par_per, par_stats
     
     def get_platform_stats(self,id):
         '''
         get statistics based on `altimetry.data.hydro_data.id`
         '''
-        par_list, valid, per_valid, fname, trange, extent, N, avail_par, avail_par_per = self.get_stats(self.id == id)
-        return (fname,trange,extent,N,avail_par,avail_par_per)
+        par_list, valid, per_valid, fname, trange, extent, N, avail_par, avail_par_per, par_stats = self.get_stats(self.id == id)
+        return (fname,trange,extent,N,avail_par,avail_par_per, par_stats)
 
     def get_object_stats(self):
         '''
@@ -855,7 +1005,9 @@ class hydro_data(object):
         self.message(0, '\t-> from : '+' - '.join(map(str,stats[1][0])))
         self.message(0, '\t-> extent : ['+', '.join(['{0:.1f}'.format(x) for x in stats[2]])+']')
         self.message(0, '\t-> size : {0} pts'.format(stats[3]))
-        self.message(0, '\t-> variables :'+', '.join(['{0}({1:.0f} %)'.format(i[0],i[1]) for i in zip(*(stats[4],stats[5]))])+']')
+        self.message(0, '\t-> variables : [%s]' % ', '.join(['{0}({1:.0f} %)'.format(i[0],i[1]) for i in zip(*(stats[4],stats[5]))])+']')
+        for p in avail_par :
+			self.message(0,'\t\to %s : {%s}' % (p,','.join(["%s:%f" % (s,stats[6][s]) for s in stats[6].keys()])))
         
     def map(self, flag=None,  fname=None, zoom=False, pmap=None, show=True, **kwargs):
         '''
@@ -883,7 +1035,7 @@ class hydro_data(object):
         outputs a summary of the whole current dataset
         """
         
-        par_list, valid, per_valid, fname, trange, extent, N, avail_par, avail_par_per = self.get_object_stats()
+        par_list, valid, per_valid, fname, trange, extent, N, avail_par, avail_par_per, par_stats = self.get_object_stats()
         
         #Print parameter list
         self.message(0, '\n\t  DATA SUMMARY')
@@ -894,8 +1046,9 @@ class hydro_data(object):
         self.message(0, '\tSpatial extension: ['+', '.join(['{0:.1f}'.format(x) for x in extent])+']')
         self.message(0, '\tRecords : [{0}]'.format(N))
         self.message(0, '\tAvailable parameters : '+', '.join(map(str,self.par_list))) 
-        self.message(0, '\tParam summary :'+', '.join(['{0}({1:.0f} %)'.format(i[0],i[1]) for i in zip(*(avail_par,avail_par_per))])+']')
-
+        self.message(0, '\tParam summary : [%s]' % ', '.join(['{0}({1:.0f} %)'.format(i[0],i[1]) for i in zip(*(avail_par,avail_par_per))]))
+        for p in avail_par :
+			self.message(0,'\t\to %s : {%s}' % (p,','.join(["%s:%s" % (s,par_stats[p][s]) for s in par_stats[p].keys()])))
         
         if isinstance(fig,str) :
             ffig = fig+'dataset.png'
@@ -1215,11 +1368,14 @@ class hydro_data(object):
     def load_ncVar(self,varName,nc=None,**kwargs):
         return load_ncVar_v2(varName, nc=self._ncfile, **kwargs)
     
-    def ncstruct(self):
+    def ncstruct(self,**kwargs):
         '''
         returns a data structure (dict) of the dataset.
+        
+        :keyword params: Add this keyword to provide a list of variables to export. Default : all variables contained is self.par_list
         '''
-        par_list = self.par_list.tolist()
+        par_list = kwargs.get('params',self.par_list.tolist())
+        
         dimStr=self._dimensions
         dimlist = dimStr.keys()
         outStr = OrderedDict({'_dimensions':dimStr})
@@ -1232,7 +1388,7 @@ class hydro_data(object):
             curDim=getattr(self.__getattribute__(d),'_dimensions',None)
             attributes = [a for a in self.__getattribute__(d).__dict__.keys() if not a.startswith('_')]
 #            attributes = np.append(attributes,'_dimensions')
-            outStr[d]={'_dimensions':self.__getattribute__(d)._dimensions}
+            outStr[d]={'_dimensions':curDim if curDim else self._dimensions}
             outStr[d].update({'data':self.__getattribute__(d)})
             for a in attributes : outStr[d].update({a:self.__getattribute__(d).__getattribute__(a)})
         
@@ -1244,9 +1400,9 @@ class hydro_data(object):
         
         :keyword kwargs: additional arguments are passed to :meth:`altimetry.tools.nctools.nc.write`
         '''
-        obj=ncobj(verbose=self.verbose,limit=self.limit,use_local_dims=True)
+        obj=ncobj(verbose=kwargs.pop('verbose',self.verbose),limit=self.limit,use_local_dims=True)
         ncalti=self.ncstruct() #Get an netcdf structure from data
-        res=obj.write(ncalti,filename,clobber=clobber,**kwargs) #Save processed datase
+        return obj.write(ncalti,filename,clobber=clobber,**kwargs) #Save processed datase
     
     def push_nc(self,*args,**kwargs):
         '''
@@ -1844,4 +2000,4 @@ def read_cnv(self,filename,**kwargs):
         ndims=np.size(sz)
                 
         return {'_dimensions':{'_ndims':ndims,'nbpoints':sz[0]},'lon':lon,'lat':lat,'date':date,'id':id,'depth':depth,'pres':pres,'temp':temp,'psal':psal}  
-   
\ No newline at end of file
+   
